@@ -67,6 +67,7 @@
 #include "tcp.h"
 #include "udp.h"
 #include "fdpass.h"
+#include "osfp.h"
 #include "pyextend.h"
 
 ssize_t atomicio(ssize_t (*)(), int, void *, size_t);
@@ -204,6 +205,15 @@ cmd_environment(struct template *tmpl, struct tuple *hdr)
 {
 	char line[256];
 	struct addr addr;
+	struct ip_hdr ip;
+	char *os_name;
+
+	/* Determine the remote operating system */
+	ip.ip_src = hdr->ip_src;
+	os_name = honeyd_osfp_name(&ip);
+	if (os_name != NULL) {
+		setenv("HONEYD_REMOTE_OS", os_name, 1);
+	}
 
 	if (tmpl->person != NULL) {
 		snprintf(line, sizeof(line), "%s", tmpl->person->name);
@@ -313,19 +323,25 @@ cmd_setpriv(struct template *tmpl)
 	extern gid_t honeyd_gid;
 	uid_t uid = honeyd_uid;
 	gid_t gid = honeyd_gid;
+	int nofiles = 30;
 	struct rlimit rl;
 
+	/* Set our own priority low */
+	setpriority(PRIO_PROCESS, 0, 10);
+
 	if (tmpl->uid)
 		uid = tmpl->uid;
 	if (tmpl->gid)
 		gid = tmpl->gid;
+	if (tmpl->max_nofiles)
+		nofiles = tmpl->max_nofiles;
 
 	cmd_droppriv(uid, gid);
 
 	/* Raising file descriptor limits */
-	rl.rlim_cur = rl.rlim_max = 24;
+	rl.rlim_cur = rl.rlim_max = nofiles;
 	if (setrlimit(RLIMIT_NOFILE, &rl) == -1)
-		err(1, "setrlimit");
+		err(1, "setrlimit: %d", nofiles);
 
 	return (0);
 }
