@@ -48,6 +48,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <syslog.h>
 #include <unistd.h>
 #include <dnet.h>
 #ifdef HAVE_TIME_H
@@ -70,9 +71,14 @@
 #include "parser.h"
 #include "ethernet.h"
 #include "arp.h"
+#include "pool.h"
+#include "dhcpclient.h"
 
 struct templtree templates;
 
+/* Counter for addresses in 169.254/16 that we assign for DHCP */
+static uint16_t privip_counter = 1;
+
 int
 templ_compare(struct template *a, struct template *b)
 {
@@ -87,10 +93,12 @@ port_compare(struct port *a, struct port *b)
 	int diff;
 
 	diff = a->proto - b->proto;
-	if (diff)
+	if (diff) {
 		return (diff);
-
-	return ((int)a->number - (int)b->number);
+	} else {
+		/* safe because number in uint16_t */
+		return ((int)a->number - (int)b->number);
+	}
 }
 
 SPLAY_PROTOTYPE(porttree, port, node, port_compare);
@@ -169,6 +177,7 @@ template_find_best(const char *addr, const struct ip_hdr *ip, u_short iplen)
 struct template *
 template_create(const char *name)
 {
+	extern rand_t *honeyd_rand;
 	struct template *tmpl;
 
 	if (template_find(name))
@@ -192,26 +201,67 @@ template_create(const char *name)
 	/* Crank ref counter */
 	tmpl->refcnt++;
 
+	/* Create a drift */
+	tmpl->drift = 1.0 + (rand_uint16(honeyd_rand) % 140 - 70)/100000.0;
+
 	return (tmpl);
 }
 
-/* Removes all configured templates from the system, so that the
+int
+template_iterate(int (*f)(struct template *, void *), void *arg)
+{
+	struct template *tmpl;
+
+	SPLAY_FOREACH(tmpl, templtree, &templates) {
+		if ((*f)(tmpl, arg) == -1)
+			return (-1);
+	}
+
+	return (0);
+}
+
+/*
+ * Removes all configured templates from the system, so that the
  * configuration file can be re-read.
  */
 
 void
-template_free_all(void)
+template_free_all(int how)
 {
 	struct template *tmpl;
 
 	while ((tmpl = SPLAY_ROOT(&templates)) != NULL) {
 		SPLAY_REMOVE(templtree, &templates, tmpl);
-		template_free(tmpl);
+		if (how == TEMPLATE_FREE_REGULAR)
+			template_free(tmpl);
+		else if (!(tmpl->flags & TEMPLATE_DYNAMIC_CHILD))
+			template_deallocate(tmpl);
 	}
 }
 
 /* Remove a template from the system */
 
+void
+template_remove(struct template *tmpl)
+{
+	/* Remove ourselves from the searchable index */
+	if (template_find(tmpl->name) == tmpl)
+		SPLAY_REMOVE(templtree, &templates, tmpl);
+}
+
+/* Insert a template into the system */
+
+int
+template_insert(struct template *tmpl)
+{
+	/* Insert ourselves into the searchable index */
+	if (template_find(tmpl->name) != NULL)
+		return (-1);
+	SPLAY_INSERT(templtree, &templates, tmpl);
+
+	return (0);
+}
+
 void
 template_deallocate(struct template *tmpl)
 {
@@ -242,10 +292,21 @@ template_deallocate(struct template *tmpl)
 
 	if (tmpl->ethernet_addr != NULL) {
 		struct arp_req *req = arp_find(tmpl->ethernet_addr);
-		arp_free(req);
+		/*
+		 * Templates that are not bound to IP addresses do not
+		 * have an associated arp request object.
+		 */
+		if (req != NULL)
+			arp_free(req);
 		free(tmpl->ethernet_addr);
 	}
 
+	if (tmpl->dhcp_req != NULL) {
+		dhcp_release(tmpl);
+		dhcp_abort(tmpl);
+		free(tmpl->dhcp_req);
+	}
+
 	free(tmpl->name);
 	free(tmpl);
 }
@@ -399,8 +460,42 @@ template_remove_subsystem(struct template *tmpl, struct subsystem *sub)
 	free(container);
 }
 
+void
+template_post_arp(struct template *tmpl, struct addr *ipaddr)
+{
+	struct arp_req *req;
+
+	/* Register this mac address as our own */
+	req = arp_new(tmpl->inter, NULL, NULL, ipaddr, tmpl->ethernet_addr);
+	if (req == NULL)
+		errx(1, "%s: cannot create arp entry");
+		
+	req->flags |= ARP_INTERNAL;
+	req->owner = tmpl;
+}
+
+void
+template_remove_arp(struct template *tmpl)
+{
+	struct arp_req *arp;
+
+	if (tmpl->ethernet_addr == NULL)
+		return;
+
+	/* Check if we have an ARP entry */
+	arp = arp_find(tmpl->ethernet_addr);
+	assert(arp != NULL);
+	arp_free(arp);
+}
+
+/*
+ * When the interface is specified, we do not need to do an address lookup
+ * for a corresponding interface.  We use the interface for DHCP.
+ */
+
 struct template *
-template_clone(const char *newname, const struct template *tmpl, int start)
+template_clone(const char *newname, const struct template *tmpl, 
+    struct interface *inter, int start)
 {
 	struct subsystem_container *container;
 	struct condition *condition;
@@ -430,21 +525,19 @@ template_clone(const char *newname, const struct template *tmpl, int start)
 		/* 
 		 * This template wants to have its own ethernet address,
 		 * so we need to register it with our arp code.
+		 *
+		 * DHCP templates get a temporary IP address assigned.
 		 */
 		if (isipaddr) {
-			struct arp_req *req;
-
-			newtmpl->inter = interface_find_responsible(&addr);
-			if (newtmpl->inter == NULL)
-				errx(1, "%s: cannot find interface");
+			if (inter == NULL) {
+				inter = interface_find_responsible(&addr);
+				if (inter == NULL)
+					errx(1, "%s: cannot find interface");
+			}
+			newtmpl->inter = inter;
 
 			/* Register this mac address as our own */
-			req = arp_new(newtmpl->inter, NULL, NULL, &addr,
-			    newtmpl->ethernet_addr);
-			if (req == NULL)
-				errx(1, "%s: cannot create arp entry");
-
-			req->flags |= ARP_INTERNAL;
+			template_post_arp(newtmpl, &addr);
 		}
 	}
 
@@ -455,10 +548,14 @@ template_clone(const char *newname, const struct template *tmpl, int start)
 	newtmpl->timestamp = tmpl->timestamp;
 	newtmpl->uid = tmpl->uid;
 	newtmpl->gid = tmpl->gid;
+	newtmpl->max_nofiles = tmpl->max_nofiles;
 	newtmpl->drop_inrate = tmpl->drop_inrate;
 	newtmpl->drop_synrate = tmpl->drop_synrate;
 	newtmpl->flags = tmpl->flags;
 
+	/* We need to remove this when cloning */
+	newtmpl->flags &= ~TEMPLATE_DYNAMIC_CHILD;
+
 	/* Clone dynamics */
 	TAILQ_FOREACH(condition, &tmpl->dynamic, next) {
 		if (template_insert_dynamic(newtmpl,
@@ -595,12 +692,13 @@ template_insert_dynamic(struct template *tmpl, struct template *child,
 
 	/* Con up a new template name */
 	snprintf(newname, sizeof(newname), "%s_%s", tmpl->name, child->name);
-	if ((cond->tmpl = template_clone(newname, child, 0)) == NULL) {
+	if ((cond->tmpl = template_clone(newname, child, NULL, 0)) == NULL) {
 		fprintf(stderr, "Failed to clone %s from %s\n",
 		    newname, child->name);
 		free(cond);
 		return (-1);
 	}
+	cond->tmpl->flags |= TEMPLATE_DYNAMIC_CHILD;
 
 	TAILQ_INSERT_TAIL(&tmpl->dynamic, cond, next);
 
@@ -615,3 +713,183 @@ template_insert_dynamic(struct template *tmpl, struct template *child,
 
 	return (0);
 }
+
+int
+template_get_dhcp_address(struct addr *addr)
+{
+	static char address[16];
+
+	snprintf(address, sizeof(address),
+	    "169.254.%d.%d", privip_counter / 256 + 1, privip_counter % 256);
+
+	if (++privip_counter > 255 * 255)
+		errx(1, "%s: out of temporary IP addresses", __func__);
+
+	return (addr_aton(address, addr));
+}
+
+/* Everything is unittest related below this */
+
+void
+template_delay_cb(int fd, short which, void *arg)
+{
+	extern struct pool *pool_pkt;
+	extern struct pool *pool_delay;
+	struct delay *delay = arg;
+	struct ip_hdr *ip = delay->ip;
+	struct template *tmpl = delay->tmpl;
+	u_int iplen = delay->iplen;
+
+	if (ip->ip_ttl &&
+	    !(delay->flags & (DELAY_UNREACH|DELAY_EXTERNAL|DELAY_TUNNEL|DELAY_ETHERNET))) {
+		struct addr addr;
+
+		template_free(tmpl);
+
+		addr_pack(&addr, ADDR_TYPE_IP, IP_ADDR_BITS,
+		    &ip->ip_dst, IP_ADDR_LEN);
+
+		/* Internal delivery */
+		tmpl = template_find_best(addr_ntoa(&addr), ip, iplen);
+		tmpl = template_ref(tmpl);
+
+		/* No Check for fragmentation */
+		honeyd_dispatch(tmpl, ip, iplen);
+
+	}
+
+	if (delay->flags & DELAY_FREEPKT)
+		pool_free(pool_pkt, ip);
+	template_free(tmpl);
+
+	if (delay->flags & DELAY_NEEDFREE)
+		pool_free(pool_delay, delay);
+}
+
+void
+template_test_parse_error(char *line, struct evbuffer *evbuf)
+{
+	char *p = EVBUFFER_DATA(evbuf);
+	size_t off = EVBUFFER_LENGTH(evbuf);
+	p[off - 1] = '\0';
+	errx(1, "parse_line \"%s\" failed: %s", line, p);
+}
+
+#define MAKE_CONFIG(x)	do { \
+	char *p = (x); \
+	if (parse_line(evbuf, p) == -1) \
+		template_test_parse_error(p, evbuf); \
+} while (0)
+	
+int
+template_test_add(struct evbuffer *evbuf, struct addr *addr, int count)
+{
+	char line[128];
+	int i;
+
+	for (i = 0; i < count; i++) {
+		evbuffer_drain(evbuf, -1);
+		
+		addr->addr_ip = htonl(ntohl(addr->addr_ip) + 1);
+		snprintf(line, sizeof(line), "bind %s template",
+		    addr_ntoa(addr));
+		MAKE_CONFIG(line);
+	}
+
+	return (count);
+}
+
+void
+template_test_measure(int count)
+{
+	extern rand_t *honeyd_rand;
+	u_char pkt[1500];
+	struct interface inter;
+	struct pcap_pkthdr pkthdr;
+	struct ip_hdr *ip = (struct ip_hdr *)pkt;
+	struct tcp_hdr *tcp = (struct tcp_hdr *)(ip + 1);
+	struct addr src, dst;
+	uint16_t *snib;
+	struct timeval tv_start, tv_end;
+	double msperpkt;
+	int j;
+	uint16_t iplen = sizeof(*ip) + sizeof(*tcp);
+
+	memset(&inter, 0, sizeof(struct interface));
+	memset(&pkthdr, 0, sizeof(struct pcap_pkthdr));
+	pkthdr.caplen = sizeof(pkt);
+		    
+	addr_pton("10.0.0.0", &dst);
+	addr_pton("10.1.0.0", &src);
+
+	snib = (uint16_t *)(&src.addr_data8[2]);
+
+	gettimeofday(&tv_start, NULL);
+	for (j = 0; j < 80000; j++) {
+		*snib = rand_uint16(honeyd_rand);
+		dst.addr_ip  = htonl(rand_uint32(honeyd_rand) % count);
+		dst.addr_data8[0] = 10;
+
+		tcp_pack_hdr(tcp,
+		    rand_uint16(honeyd_rand), 23,
+		    0x100000, 0x00000, TH_SYN, 32768, 0);
+		ip_pack_hdr(pkt, 0, iplen, rand_uint16(honeyd_rand),
+		    0, 64,
+		    IP_PROTO_TCP, src.addr_ip, dst.addr_ip);
+		ip_checksum(pkt, iplen);
+
+		honeyd_recv_cb(&inter, &pkthdr, pkt);
+	}
+	gettimeofday(&tv_end, NULL);
+	timersub(&tv_end, &tv_start, &tv_end);
+	msperpkt = (double)(tv_end.tv_sec * 1000 + tv_end.tv_usec / 1000)
+	    / (double) j;
+	fprintf(stderr, "\t\t%7d templates: %.4f ms per packet\n",
+	    count, msperpkt);
+}
+
+void
+template_packet_test(void)
+{
+	extern void (*honeyd_delay_callback)(int, short, void *);
+	void (*old)(int, short, void *) = honeyd_delay_callback;
+	struct evbuffer *evbuf = evbuffer_new();
+	struct addr addr;
+	int i, count;
+
+	/* Set our own delay callback */
+	honeyd_delay_callback = template_delay_cb;
+
+	/* Create configuration */
+	MAKE_CONFIG("create template");
+	MAKE_CONFIG("add template tcp port 23 reset");
+
+	addr_pton("10.0.0.0", &addr);
+
+	count = template_test_add(evbuf, &addr, 1);
+	template_test_measure(count);
+	for (i = 0; i < 5; i++) {
+		count += template_test_add(evbuf, &addr,
+		    count == 1 ? 999 : 1000);
+		template_test_measure(count);
+	}
+	for (i = 0; i < 50; i++) {
+		count += template_test_add(evbuf, &addr, 5000);
+		template_test_measure(count);
+	}
+
+	honeyd_delay_callback = old;
+
+	evbuffer_free(evbuf);
+
+	fprintf(stderr, "\t%s: OK\n", __func__);
+}
+
+
+void
+template_test(void)
+{
+	setlogmask(LOG_UPTO(LOG_NOTICE));
+
+	template_packet_test();
+}
