@@ -43,9 +43,9 @@
 
 #define HONEYD_MAX_CONNECTS	32000
 
-#define HONEYD_CLOSE_WAIT	10
-#define HONEYD_SYN_WAIT		30
-#define HONEYD_IDLE_TIMEOUT	60
+#define HONEYD_CLOSE_WAIT	60
+#define HONEYD_SYN_WAIT		60
+#define HONEYD_IDLE_TIMEOUT	300
 #define HONEYD_DFL_TTL		64
 #define HONEYD_UDP_WAIT		60
 #define HONEYD_MAX_SOFTERRS	3	   /* Softerrors to state free*/
@@ -54,6 +54,20 @@
 
 #define HONEYD_ADDR_MASK        0xFFFFFF00 /* for ICMP address mask replies */
 
+struct config {
+	char *config;	 /* Name of configuration file */
+	char *pers;
+	char *xprobe;
+	char *assoc;
+	char *osfp;
+};
+
+struct count;
+struct stats_network {
+	struct count *input_bytes;
+	struct count *output_bytes;
+};
+
 struct delay {
 	struct event timeout;
 
@@ -120,6 +134,7 @@ SPLAY_HEAD(porttree, port);
 
 struct tuple {
 	SPLAY_ENTRY(tuple) node;
+	TAILQ_ENTRY(tuple) next;
 
 	ip_addr_t ip_src;
 	ip_addr_t ip_dst;
@@ -132,9 +147,14 @@ struct tuple {
 	uint32_t received;
 	uint32_t sent;
 
+	struct event timeout;
+
 	int local;	/* locally initiated */
 };
 
+SPLAY_HEAD(tree, tuple);
+TAILQ_HEAD(conlru, tuple);
+
 struct command {
 	pid_t pid;
 	int pfd;
@@ -196,7 +216,6 @@ struct tcp_con {
 
 	u_short retrans_time;
 
-	struct event timeout;
 	struct event retrans_timeout;
 
 	struct port *port;		/* used if bound to sub system */
@@ -226,8 +245,6 @@ struct udp_con {
 	TAILQ_HEAD(bufferq, conbuffer) incoming;
 	int nincoming;
 
-	struct event timeout;
-
 	int softerrors;		/* ICMP unreachables for this state */
 
 	struct port *port;
@@ -274,6 +291,10 @@ struct icmp_msg_inforeply { /* Our definition */
 #define TCP_BYTESINFLIGHT(x)	(x)->poff
 #define TCP_MAX_INFLIGHT	4096
 
+/* Iterate over all active connections */
+int tuple_iterate(struct conlru *, int (*f)(struct tuple *, void *), void *);
+struct tuple *tuple_find(struct tree *, struct tuple *);
+
 void honeyd_ip_send(u_char *, u_int);
 void honeyd_dispatch(struct template *, struct ip_hdr *, u_short);
 char *honeyd_contoa(const struct tuple *);
@@ -310,6 +331,7 @@ int tcp_setupconnect(struct tcp_con *);
 void generic_timeout(struct event *, int);
 
 /* Network protocol related prototypes */
+int conhdr_compare(struct tuple *, struct tuple *);
 
 void tcp_free(struct tcp_con *);
 int tcp_send(struct tcp_con *, uint8_t, u_char *, u_int);
@@ -343,6 +365,7 @@ void icmp_timestamp_reply(struct template *, struct ip_hdr *,
 /* Utility functions */
 int addr_contained(struct addr *, struct addr *);
 char *strrpl(char *, size_t, char *, char *);
+char *fgetln(FILE *, size_t *);
 
 void honeyd_use_uid(uid_t);
 void honeyd_use_gid(gid_t);
