@@ -1,5 +1,35 @@
 /*
  * Copyright (c) 2003 Niels Provos <provos@citi.umich.edu>
+ * All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+/*
  * Copyright (c) 2002 Marius Aamodt Eriksen <marius@monkey.org>
  * All rights reserved.
  *
@@ -69,7 +99,7 @@
 #include "subsystem.h"
 #include "fdpass.h"
 
-#define DEBUG 1
+#undef DEBUG
 #ifdef DEBUG
 #define DPRINTF(x) fprintf x
 #else
@@ -227,7 +257,6 @@ newsock_fd(int domain, int type, int protocol)
 		return (NULL);
 	}
 
-
 	if ((nfd = new_fd(pair[0])) == NULL) {
 		(*libc_close)(pair[0]);
 		(*libc_close)(pair[1]);
@@ -347,8 +376,8 @@ socket(int domain, int type, int protocol)
 	if (domain != AF_INET)
 		return ((*libc_socket)(domain, type, protocol));
 
-	DPRINTF((stderr, "Attemping to create socket: %d %d %d\n",
-	    domain, type, protocol));
+	DPRINTF((stderr, "%s: Attempting to create socket: %d %d %d\n",
+	    __func__, domain, type, protocol));
 
 	nfd = newsock_fd(domain, type, protocol);
 	if (nfd == NULL) {
@@ -649,6 +678,17 @@ sendto(int sock, const void *buf, size_t len, int flags,
 	if (nfd == NULL)
 		return ((*libc_sendto)(sock, buf, len, flags, to, tolen));
 
+	/*
+	 * UDP sockets can be sendto when they are not yet connected.
+	 * However, Honeyd requires UDP sockets to be connected.  This can
+	 * cause problems when an application uses one UDP socket to talk
+	 * to multiple hosts.
+	 */
+	if (!(nfd->flags & FD_CONNECTED) && nfd->protocol == IPPROTO_UDP) {
+		DPRINTF((stderr, "%s : false connect is needed !\n",__func__));
+		connect(sock,to,tolen);
+	}
+
 	ret = (*libc_sendto)(sock, buf, len, flags, NULL, 0);
 
 	DPRINTF((stderr, "%s: called: %d: %p,%d -> %d (%s)\n", __func__,
