@@ -95,27 +95,26 @@ struct honeyd_packet_hook
 
 TAILQ_HEAD(hooksq, honeyd_packet_hook);
 
-/* Two arrays if hook tailqueues, each with HD_HOOKS_LAST elements,
+/*
+ * Arrays of hook tailqueues, each with HD_HOOKS_LAST elements,
  * indexed using the HD_HOOKS_xxx constants:
  */
-struct hooksq  *in_hooks;
-struct hooksq  *out_hooks;
+struct hooksq  *dir_hooks[HD_DIR_MAX];
 
 void    
 hooks_init(void)
 {
-	int i;
+	int i, j;
 
-	in_hooks  = malloc(HD_HOOKS_LAST * sizeof(struct hooksq));
-	out_hooks = malloc(HD_HOOKS_LAST * sizeof(struct hooksq));
-
-	if (in_hooks == NULL || out_hooks == NULL)
-		err(1, "%s: malloc", __FUNCTION__);
-		
-	for (i = 0; i < HD_HOOKS_LAST; i++) {
-		TAILQ_INIT(&in_hooks[i]);
-		TAILQ_INIT(&out_hooks[i]);
+	for (i = 0; i < HD_DIR_MAX; i++) {
+		dir_hooks[i] = malloc(HD_HOOKS_LAST * sizeof(struct hooksq));
+		if (dir_hooks[i] == NULL)
+			err(1, "%s: malloc", __func__);
 	}
+
+	for (i = 0; i < HD_HOOKS_LAST; i++)
+		for (j = 0; j < HD_DIR_MAX; j++)
+			TAILQ_INIT(&dir_hooks[j][i]);
 }
 
 void    
@@ -135,7 +134,7 @@ hooks_add_packet_hook(int protocol, HD_Direction dir,
 	hook->callback  = callback;
 	hook->user_data = user_data;
 
-	hooks = (dir == HD_INCOMING) ? in_hooks : out_hooks;
+	hooks = dir_hooks[dir];
 
 	switch (protocol) {
 	case IP_PROTO_TCP:
@@ -179,7 +178,7 @@ hooks_remove_packet_hook(int protocol, HD_Direction dir,
 	if (callback == NULL)
 		return;
 	
-	hooks = (dir == HD_INCOMING) ? in_hooks : out_hooks;
+	hooks = dir_hooks[dir];
 
 	switch (protocol) {
 	case IP_PROTO_TCP:
@@ -201,7 +200,7 @@ hooks_remove_packet_hook(int protocol, HD_Direction dir,
 
 
 void    
-hooks_dispatch(int protocol, HD_Direction dir,
+hooks_dispatch(int protocol, HD_Direction dir, struct tuple *conhdr,
     u_char *packet_data, u_int packet_len)
 {
 	struct hooksq *hooks;
@@ -210,26 +209,30 @@ hooks_dispatch(int protocol, HD_Direction dir,
 	if (packet_data == NULL)
 		return;
 	
-	hooks = (dir == HD_INCOMING) ? in_hooks : out_hooks;
+	hooks = dir_hooks[dir];
 
 	switch (protocol) {
 	case IP_PROTO_TCP:
 		TAILQ_FOREACH(hook, &hooks[HD_HOOKS_TCP], next)
-		    hook->callback(packet_data, packet_len, hook->user_data);
+		    hook->callback(conhdr, packet_data, packet_len,
+			hook->user_data);
 		break;
 		
 	case IP_PROTO_UDP:
 		TAILQ_FOREACH(hook, &hooks[HD_HOOKS_UDP], next)
-		    hook->callback(packet_data, packet_len, hook->user_data);
+		    hook->callback(conhdr, packet_data, packet_len,
+			hook->user_data);
 		break;
 		
 	case IP_PROTO_ICMP:
 		TAILQ_FOREACH(hook, &hooks[HD_HOOKS_ICMP], next)
-		    hook->callback(packet_data, packet_len, hook->user_data);
+		    hook->callback(conhdr, packet_data, packet_len,
+			hook->user_data);
 		break;
 		
 	default:
 		TAILQ_FOREACH(hook, &hooks[HD_HOOKS_OTHER], next)
-		    hook->callback(packet_data, packet_len, hook->user_data);
+		    hook->callback(conhdr, packet_data, packet_len,
+			hook->user_data);
 	}  
 }
