@@ -138,7 +138,7 @@ ip_fragment_free(struct fragment *tmp)
 {
 	struct fragent *ent;
 
-	timeout_del(&tmp->timeout);
+	evtimer_del(&tmp->timeout);
 
 	SPLAY_REMOVE(fragtree, &fragments, tmp);
 	TAILQ_REMOVE(&fraglru, tmp, next);
@@ -205,8 +205,8 @@ ip_fragment_new(ip_addr_t src, ip_addr_t dst, u_short id, enum fragpolicy pl)
 	tmp->fragp = pl;
 
 	TAILQ_INIT(&tmp->fraglist);
-	timeout_set(&tmp->timeout, ip_fragment_timeout, tmp);
-	timeout_add(&tmp->timeout, &tv);
+	evtimer_set(&tmp->timeout, ip_fragment_timeout, tmp);
+	evtimer_add(&tmp->timeout, &tv);
 
 	SPLAY_INSERT(fragtree, &fragments, tmp);
 	TAILQ_INSERT_HEAD(&fraglru, tmp, next);
