@@ -36,10 +36,24 @@
 void pyextend_init(void);
 void pyextend_exit(void);
 
+#define PYEXTEND_MAX_REQUEST_SIZE	16384
+
+void pyextend_webserver_init(int port, char *root_dir);
+void pyextend_webserver_exit(void);
+void pyextend_webserver_verify_setup(const char *);
+void pyextend_webserver_fix_permissions(const char *, uid_t, gid_t);
+
 int pyextend_connection_start(struct tuple *, struct command *, void *arg,
     void *pye);
 struct pystate;
 void pyextend_connection_end(struct pystate *);
 void *pyextend_load_module(const char *);
 
+struct evbuffer;
+struct pyextend_request {
+	int fd;
+	struct addr src;
+	struct bufferevent *evb;
+};
+
 #endif /* _PYEXTEND_ */
