@@ -59,6 +59,7 @@
 #include "honeyd.h"
 #include "tcp.h"
 #include "log.h"
+#include "hooks.h"
 
 struct callback cb_tcp = {
 	cmd_tcp_read, cmd_tcp_write, cmd_tcp_eread, cmd_tcp_connect_cb
@@ -108,6 +109,9 @@ tcp_add_readbuf(struct tcp_con *con, u_char *dat, u_int datlen)
 {
 	int space;
 
+	hooks_dispatch(IP_PROTO_TCP, HD_INCOMING_STREAM, &con->conhdr,
+	    dat, datlen);
+
 	if (con->cmd_pfd == -1)
 		return (datlen);
 
