@@ -59,6 +59,7 @@
 #include "honeyd.h"
 #include "udp.h"
 #include "log.h"
+#include "hooks.h"
 
 struct callback cb_udp = {
 	cmd_udp_read, cmd_udp_write, cmd_udp_eread, cmd_udp_connect_cb
@@ -92,7 +93,10 @@ cmd_udp_eread(int fd, short which, void *arg)
 void
 udp_add_readbuf(struct udp_con *con, u_char *dat, u_int datlen)
 {
-	struct buffer *buf;
+	struct conbuffer *buf;
+
+	hooks_dispatch(IP_PROTO_UDP, HD_INCOMING_STREAM, &con->conhdr,
+	    dat, datlen);
 
 	if (con->cmd_pfd == -1)
 		return;
@@ -100,7 +104,7 @@ udp_add_readbuf(struct udp_con *con, u_char *dat, u_int datlen)
 	if (con->nincoming >= MAX_UDP_BUFFERS)
 		return;
 
-	buf = malloc(sizeof(struct buffer));
+	buf = malloc(sizeof(struct conbuffer));
 	if (buf == NULL)
 		return;
 	buf->buf = malloc(datlen);
@@ -145,7 +149,7 @@ void
 cmd_udp_write(int fd, short which, void *arg)
 {
 	struct udp_con *con = arg;
-	struct buffer *buf;
+	struct conbuffer *buf;
 	ssize_t len;
 	
 	buf = TAILQ_FIRST(&con->incoming);
