@@ -66,6 +66,12 @@
 #include "config.h"
 #endif
 
+#include <sys/queue.h>
+#include <sys/tree.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
 #include <err.h>
 #include <errno.h>
 #include <stdio.h>
@@ -73,9 +79,13 @@
 #include <string.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <fcntl.h>
 
 #include <pcap.h>
 #include <dnet.h>
+#include <event.h>
+
+#include "honeyd.h"
 
 int
 pcap_dloff(pcap_t *pd)
@@ -172,6 +182,30 @@ addr_contained(struct addr *net, struct addr *addr)
 	return (1);
 }
 
+char *
+strnsep(char **line, char *delim)
+{
+	char *ret, *p;
+
+	if (line == NULL)
+		return (NULL);
+
+	ret = *line;
+	if (ret == NULL)
+		return (NULL);
+
+	p = strpbrk(ret, delim);
+	if (p == NULL) {
+		*line = NULL;
+		return (ret);
+	}
+
+	*line = p + strspn(p, delim);
+	*p = '\0';
+
+	return (ret);
+}
+
 #ifndef HAVE_FGETLN
 char *
 fgetln(FILE *stream, size_t *len)
@@ -186,3 +220,133 @@ fgetln(FILE *stream, size_t *len)
 	return (buf);
 }
 #endif
+
+/* Either connect or bind */
+
+int
+make_socket_ai(int (*f)(int, const struct sockaddr *, socklen_t), int type,
+    struct addrinfo *ai)
+{
+        struct linger linger;
+        int fd, on = 1;
+
+        /* Create listen socket */
+        fd = socket(AF_INET, type, 0);
+        if (fd == -1) {
+                warn("socket");
+                return (-1);
+        }
+
+        if (fcntl(fd, F_SETFL, O_NONBLOCK) == -1) {
+                warn("fcntl(O_NONBLOCK)");
+                goto out;
+        }
+
+        if (fcntl(fd, F_SETFD, 1) == -1) {
+                warn("fcntl(F_SETFD)");
+                goto out;
+        }
+
+	if (type == SOCK_STREAM) {
+		setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE,
+		    (void *)&on, sizeof(on));
+		setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
+		    (void *) &on, sizeof(on));
+#ifdef SO_REUSEPORT
+		setsockopt(fd, SOL_SOCKET, SO_REUSEPORT,
+		    (void *) &on, sizeof(on));
+#endif
+		linger.l_onoff = 1;
+		linger.l_linger = 5;
+		setsockopt(fd, SOL_SOCKET, SO_LINGER, &linger, sizeof(linger));
+	}
+
+        if ((f)(fd, ai->ai_addr, ai->ai_addrlen) == -1) {
+		if (errno != EINPROGRESS) {
+			warn("%s", __func__);
+			goto out;
+		}
+        }
+
+	return (fd);
+
+ out:
+	close(fd);
+	return (-1);
+}
+
+int
+make_socket(int (*f)(int, const struct sockaddr *, socklen_t), int type,
+    char *address, uint16_t port)
+{
+        struct addrinfo ai, *aitop;
+        char strport[NI_MAXSERV];
+	int fd;
+	
+        memset(&ai, 0, sizeof (ai));
+        ai.ai_family = AF_INET;
+        ai.ai_socktype = type;
+        ai.ai_flags = f != connect ? AI_PASSIVE : 0;
+        snprintf(strport, sizeof (strport), "%d", port);
+        if (getaddrinfo(address, strport, &ai, &aitop) != 0) {
+                warn("getaddrinfo");
+                return (-1);
+        }
+        
+	fd = make_socket_ai(f, type, aitop);
+
+	freeaddrinfo(aitop);
+
+	return (fd);
+}
+
+#define DIFF(a,b) do { \
+	if ((a) < (b)) return -1; \
+	else if ((a) > (b)) return 1; \
+} while (0)
+
+int
+conhdr_compare(struct tuple *a, struct tuple *b)
+{
+	DIFF(a->ip_src, b->ip_src);
+	DIFF(a->ip_dst, b->ip_dst);
+	DIFF(a->sport, b->sport);
+	DIFF(a->dport, b->dport);
+
+	return (0);
+}
+
+char *
+honeyd_contoa(const struct tuple *hdr)
+{
+	static char buf[128];
+	char asrc[24], adst[24];
+	struct addr src, dst;
+	u_short sport, dport;
+	
+	addr_pack(&src, ADDR_TYPE_IP, IP_ADDR_BITS, &hdr->ip_src, IP_ADDR_LEN);
+	addr_pack(&dst, ADDR_TYPE_IP, IP_ADDR_BITS, &hdr->ip_dst, IP_ADDR_LEN);
+
+	/* For a local connection switch the address around */
+	if (hdr->local) {
+		struct addr tmp;
+
+		tmp = src;
+		src = dst;
+		dst = tmp;
+
+		sport = hdr->dport;
+		dport = hdr->sport;
+	} else {
+		sport = hdr->sport;
+		dport = hdr->dport;
+	}
+
+	addr_ntop(&src, asrc, sizeof(asrc));
+	addr_ntop(&dst, adst, sizeof(adst));
+
+	snprintf(buf, sizeof(buf), "(%s:%d - %s:%d)",
+	    asrc, sport, adst, dport);
+
+	return (buf);
+}
